import { A as emberArray } from '@ember/array';
import Ember from 'ember';
import QueryParams from '../query-params';

const { canInvoke } = Ember;

const { keys } = Object;

/**
 * Change event generated by query params changing.
 *
 * @export
 * @class ParachuteEvent
 */
export default class ParachuteEvent {
  /**
   * Creates an instance of ParachuteEvent.
   *
   * @param {string} routeName
   * @param {Ember.Controller} controller
   * @param {object} [changed={}]
   *
   * @memberof ParachuteEvent
   */
  constructor(routeName, controller, changed = {}) {
    let { queryParams, queryParamsArray } = QueryParams.metaFor(controller);
    let state = QueryParams.stateFor(controller);
    let changedKeys = emberArray(keys(changed));

    /**
     * The route the event was fired from
     */
    this.routeName = routeName;

    /**
     * All query params that have changed from this update event
     */
    this.changed = queryParamsArray.reduce(
      (changedParams, qp) => {
        if (changedKeys.includes(qp.as)) {
          changedParams[qp.key] = canInvoke(qp, 'deserialize')
            ? qp.deserialize(changed[qp.as], controller)
            : changed[qp.as];
        }
        return changedParams;
      },
      {},
      undefined
    );

    /**
     * All Query Params at this given moment
     */
    this.queryParams = QueryParams.queryParamsFor(controller);

    /**
     * Whether or not a model refresh should occur
     */
    this.shouldRefresh = emberArray(keys(this.changed)).any(
      key => queryParams[key].refresh
    );

    /**
     * All query params that are not their default
     */
    this.changes = keys(state).reduce((changes, key) => {
      if (state[key].changed) {
        changes[key] = state[key].value;
      }
      return changes;
    }, {});
  }
}
